package com.xy.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.yitter.idgen.YitIdHelper;
import com.xy.device.EnumDeviceCharging;
import com.xy.device.EnumDeviceChargingConfigType;
import com.xy.dto.DeviceChargingConfigDto;
import com.xy.dto.be.MercDto;
import com.xy.entity.DeviceChargingConfig;
import com.xy.mapper.DeviceChargingConfigMapper;
import com.xy.service.be.MercService;
import com.xy.utils.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 设备计费配置表 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-07-28
 */
@Service
@AllArgsConstructor
@Api(tags = "设备计费配置表")
public class DeviceChargingConfigServiceImpl extends ServiceImpl<DeviceChargingConfigMapper, DeviceChargingConfig> implements DeviceChargingConfigService {

    private MercService mercService;

    private DeviceAlgorithmChargingServiceImpl deviceAlgorithmChargingService;

    @PostMapping("list")
    @ApiOperation("集合查询")
    public R<List<DeviceChargingConfigDto.Vo>> list(@RequestBody @Validated DeviceChargingConfigDto.SelectList selectList) {
        MercDto.Vo merc = mercService.obj(new MercDto.Vo().setId(selectList.getMercId())).getData();
        LambdaQueryWrapper<DeviceChargingConfig> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(selectList, DeviceChargingConfig.class)
                .in(DeviceChargingConfig::getDeviceType, selectList.getDeviceTypes())
                .eq(DeviceChargingConfig::getMercCode, merc.getMercCode())
                .build();
        List<DeviceChargingConfig> list = list(lambdaQueryWrapper);
        return R.ok(Beans.copy(DeviceChargingConfigDto.Vo.class, list));
    }

    @PostMapping("listByFeeConfig")
    @ApiOperation("设备管理费用配置查询")
    public R<List<DeviceChargingConfigDto.FeeConfigVO>> listByFeeConfig(@RequestBody @Validated DeviceChargingConfigDto.ListByFeeConfigDTO dto) {
        MercDto.Vo merc = mercService.obj(new MercDto.Vo().setId(dto.getMercId())).getData();
        LambdaQueryWrapper<DeviceChargingConfig> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(dto, DeviceChargingConfig.class)
                .in(DeviceChargingConfig::getDeviceType, dto.getDeviceTypes())
                .eq(DeviceChargingConfig::getType, SysDictUtils.getValue(EnumDeviceChargingConfigType.Code.CODE.getCode(), EnumDeviceChargingConfigType.N_2.getCode(), Integer.class))
                .eq(DeviceChargingConfig::getMercCode, merc.getMercCode())
                .build();
        //默认配置 type 2预充 1赠送天数
        List<DeviceChargingConfig> list = list(lambdaQueryWrapper);
        Map<Integer, DeviceChargingConfig> typeConfigMap = MapUtil.newHashMap();
        if (CollUtil.isNotEmpty(list)) {
            typeConfigMap = list.stream().collect(Collectors.toMap(DeviceChargingConfig::getDeviceType, c -> c));
        }
        //赠送天数
        LambdaQueryWrapper<DeviceChargingConfig> lambdaQueryWrapper2 = new MybatisPlusQuery().eqWrapper(dto, DeviceChargingConfig.class)
                .in(DeviceChargingConfig::getDeviceType, dto.getDeviceTypes())
                .eq(DeviceChargingConfig::getType, SysDictUtils.getValue(EnumDeviceChargingConfigType.Code.CODE.getCode(), EnumDeviceChargingConfigType.N_1.getCode(), Integer.class))
                .eq(DeviceChargingConfig::getMercCode, merc.getMercCode())
                .build();
        List<DeviceChargingConfig> list2 = list(lambdaQueryWrapper2);
        Map<Integer, DeviceChargingConfig> typeConfigMap2 = MapUtil.newHashMap();
        if (CollUtil.isNotEmpty(list2)) {
            typeConfigMap2 = list2.stream().collect(Collectors.toMap(DeviceChargingConfig::getDeviceType, c -> c));
        }

        List<DeviceChargingConfigDto.FeeConfigVO> voList = new ArrayList<>();
        List<Integer> deviceTypes = dto.getDeviceTypes();
        for (Integer type : deviceTypes) {//默认年费，赠送天数
            DeviceChargingConfigDto.FeeConfigVO vo = new DeviceChargingConfigDto.FeeConfigVO();
            int defaultMoney = SysDictUtils.getValue(EnumDeviceCharging.Code.CODE.getCode(), String.valueOf(type), Integer.class);
            //默认赠送天数
            int giveDays = SysDictUtils.getValue(EnumDeviceCharging.Code.CODE.getCode(), EnumDeviceCharging.N_X.getCode(), Integer.class);
            if ((CollUtil.isNotEmpty(typeConfigMap) && typeConfigMap.get(type) != null) || (CollUtil.isNotEmpty(typeConfigMap2) && typeConfigMap2.get(type) != null)) {
                DeviceChargingConfig deviceChargingConfig = new DeviceChargingConfig();
                //存在自定义配置 预充
                if (CollUtil.isNotEmpty(typeConfigMap) && typeConfigMap.get(type) != null) {
                    deviceChargingConfig = typeConfigMap.get(type);
                    BeanUtil.copyProperties(deviceChargingConfig, vo);
                    vo.setDefFee(defaultMoney);
                    vo.setDefGiveDay(giveDays);
                    vo.setFee(deviceChargingConfig.getConfigValue());
                    vo.setGiveDay(giveDays);
                }

                //存在自定义配置 赠送天数
                if (CollUtil.isNotEmpty(typeConfigMap2) && typeConfigMap2.get(type) != null) {
                    if (deviceChargingConfig != null) {
                        DeviceChargingConfig deviceChargingConfig2 = typeConfigMap2.get(type);
                        vo.setDefFee(defaultMoney);
                        vo.setDefGiveDay(giveDays);
                        vo.setGiveDay(deviceChargingConfig2.getConfigValue());
                        voList.add(vo);
                    } else {
                        //未设置预充
                        deviceChargingConfig = typeConfigMap2.get(type);
                        BeanUtil.copyProperties(deviceChargingConfig, vo);
                        vo.setDefFee(defaultMoney);
                        vo.setDefGiveDay(giveDays);
                        vo.setFee(defaultMoney);
                        vo.setGiveDay(deviceChargingConfig.getConfigValue());
                        voList.add(vo);
                    }

                }


            } else {
                //不存在，则取字典默认值
                vo.setId(YitIdHelper.nextId());
                vo.setFee(defaultMoney);
                vo.setGiveDay(giveDays);
                vo.setDeviceType(type);
                vo.setMercCode(merc.getMercCode());
                vo.setDefFee(defaultMoney);
                vo.setDefGiveDay(giveDays);
                voList.add(vo);
            }

        }
        return R.ok(voList);
    }

    @PostMapping("saveOrUpdate")
    @ApiOperation("添加或修改")
    public R saveOrUpdate(@RequestBody @Validated DeviceChargingConfigDto.SaveOrUpdate saveOrUpdate) {
        LocalDateTime now = LocalDateTime.now();
        MercDto.Vo merc = mercService.obj(new MercDto.Vo().setId(saveOrUpdate.getMercId())).getData();
        List<DeviceChargingConfigDto.SaveOrUpdate.ConfigValue> values = saveOrUpdate.getConfigValues();
        for (DeviceChargingConfigDto.SaveOrUpdate.ConfigValue value : values) {
            LambdaQueryWrapper<DeviceChargingConfig> lambdaQueryWrapper = new LambdaQueryWrapper<DeviceChargingConfig>()
                    .eq(DeviceChargingConfig::getMercCode, merc.getMercCode())
                    .eq(DeviceChargingConfig::getDeviceType, saveOrUpdate.getDeviceType())
                    .eq(DeviceChargingConfig::getType, value.getType());
            DeviceChargingConfig deviceChargingConfig = getOne(lambdaQueryWrapper);
            if (deviceChargingConfig == null) {
                if (value.getConfigValue() == -1) {
                    continue;
                }
                deviceChargingConfig = new DeviceChargingConfig()
                        .setId(YitIdHelper.nextId())
                        .setMercCode(merc.getMercCode())
                        .setDeviceType(saveOrUpdate.getDeviceType())
                        .setConfigValue(value.getConfigValue())
                        .setNote(saveOrUpdate.getNote())
                        .setType(value.getType())
                        .setCreateTime(now)
                        .setUpdateTime(now);
                save(deviceChargingConfig);
            } else {
                if (value.getConfigValue() == -1) {
                    removeById(deviceChargingConfig.getId());
                    continue;
                }
                deviceChargingConfig.setConfigValue(value.getConfigValue())
                        .setNote(saveOrUpdate.getNote())
                        .setUpdateTime(now);
                updateById(deviceChargingConfig);
            }
        }
        return R.ok();
    }

    @Override
    @ApiOperation("集合查询2")
    public R<List<DeviceChargingConfigDto.Vo>> list2(DeviceChargingConfigDto.SelectList selectList) {
        MercDto.Vo merc = mercService.obj(new MercDto.Vo().setId(selectList.getMercId())).getData();
        List<String> mercCodes = new ArrayList<>();
        deviceAlgorithmChargingService.getMercCodes(merc.getMercCode(), mercCodes);
        LambdaQueryWrapper<DeviceChargingConfig> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(selectList, DeviceChargingConfig.class)
                .in(DeviceChargingConfig::getMercCode, mercCodes)
                .build()
                .orderByDesc(DeviceChargingConfig::getMercCode);
        List<DeviceChargingConfig> list = list(lambdaQueryWrapper);
        return R.ok(Beans.copy(DeviceChargingConfigDto.Vo.class, list));
    }
}
